@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: rcode.py,v 1.6 2003/06/24 02:37:00 halley Exp $
+# $Id: rcode.py,v 1.10 2003/08/02 06:08:38 halley Exp $
 
 """DNS Result Codes."""
 
@@ -47,20 +47,12 @@ _by_text = {
     'BADVERS' : BADVERS
 }
 
-_by_value = {
-    NOERROR : 'NOERROR',
-    FORMERR : 'FORMERR',
-    SERVFAIL : 'SERVFAIL',
-    NXDOMAIN : 'NXDOMAIN',
-    NOTIMP : 'NOTIMP',
-    REFUSED : 'REFUSED',
-    YXDOMAIN : 'YXDOMAIN',
-    YXRRSET : 'YXRRSET',
-    NXRRSET : 'NXRRSET',
-    NOTAUTH : 'NOTAUTH',
-    NOTZONE : 'NOTZONE',
-    BADVERS : 'BADVERS'
-}
+# We construct the inverse mapping programmatically to ensure that we
+# cannot make any mistakes (e.g. omissions, cut-and-paste errors) that
+# would cause the mapping not to be a true inverse.
+
+_by_value = dict([(y, x) for x, y in _by_text.iteritems()])
+
 
 class UnknownRcode(dns.exception.DNSException):
     """Raised if an rcode is unknown."""
@@ -68,34 +60,62 @@ class UnknownRcode(dns.exception.DNSException):
 
 def from_text(text):
     """Convert text into an rcode.
-    @rtype: string"""
     
-    try:
-        v = _by_text[text.upper()]
-    except KeyError:
+    @param text: the texual rcode
+    @type text: string
+    @raises UnknownRcode: the rcode is unknown
+    @rtype: int
+    """
+
+    if text.isdigit():
+        v = int(text)
+        if v >= 0 and v <= 4095:
+            return v
+    v = _by_text.get(text.upper())
+    if v is None:
         raise UnknownRcode
     return v
 
 def from_flags(flags, ednsflags):
     """Return the rcode value encoded by flags and ednsflags.
-    @rtype: int"""
-    
-    return (flags & 0x000f) | ((ednsflags >> 20) & 0xff0)
+
+    @param flags: the DNS flags
+    @type flags: int
+    @param ednsflags: the EDNS flags
+    @type ednsflags: int
+    @raises ValueError: rcode is < 0 or > 4095
+    @rtype: int
+    """
+
+    value = (flags & 0x000f) | ((ednsflags >> 20) & 0xff0)
+    if value < 0 or value > 4095:
+        raise ValueError, 'rcode must be >= 0 and <= 4095'
+    return value
 
 def to_flags(value):
     """Return a (flags, ednsflags) tuple which encodes the rcode.
-    @rtype: (int, int) tuple"""
-    
+
+    @param value: the rcode
+    @type value: int
+    @raises ValueError: rcode is < 0 or > 4095
+    @rtype: (int, int) tuple
+    """
+
+    if value < 0 or value > 4095:
+        raise ValueError, 'rcode must be >= 0 and <= 4095'
     v = value & 0xf
-    ev = (value & 0xff0) << 20
+    ev = long(value & 0xff0) << 20
     return (v, ev)
 
 def to_text(value):
     """Convert rcode into text.
-    @rtype: string"""
+
+    @param value: the rcode
+    @type value: int
+    @rtype: string
+    """
     
-    try:
-        text = _by_value[value]
-    except KeyError:
-        raise UnknownRcode
+    text = _by_value.get(value)
+    if text is None:
+        text = str(value)
     return text
