@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: AAAA.py,v 1.13 2003/06/30 06:08:27 halley Exp $
+# $Id: AAAA.py,v 1.15 2003/08/05 06:22:26 halley Exp $
 
 import socket
 
@@ -28,10 +28,12 @@ class AAAA(dns.rdata.Rdata):
     @ivar address: an IPv6 address
     @type address: string (in the standard IPv6 format)"""
 
+    __slots__ = ['address']
+    
     def __init__(self, rdclass, rdtype, address):
         super(AAAA, self).__init__(rdclass, rdtype)
         # check that it's OK
-        junk = dns.inet.inet_pton(socket.AF_INET6, address)
+        junk = dns.inet.inet_pton(dns.inet.AF_INET6, address)
         self.address = address
 
     def to_text(self, origin=None, relativize=True, **kw):
@@ -47,16 +49,16 @@ class AAAA(dns.rdata.Rdata):
     from_text = classmethod(from_text)
 
     def to_wire(self, file, compress = None, origin = None):
-        file.write(dns.inet.inet_pton(socket.AF_INET6, self.address))
+        file.write(dns.inet.inet_pton(dns.inet.AF_INET6, self.address))
         
     def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin = None):
-        address = dns.inet.inet_ntop(socket.AF_INET6,
+        address = dns.inet.inet_ntop(dns.inet.AF_INET6,
                                      wire[current : current + rdlen])
         return cls(rdclass, rdtype, address)
 
     from_wire = classmethod(from_wire)
 
     def _cmp(self, other):
-        sa = dns.inet.inet_pton(socket.AF_INET6, self.address)
-        oa = dns.inet.inet_pton(socket.AF_INET6, other.address)
+        sa = dns.inet.inet_pton(dns.inet.AF_INET6, self.address)
+        oa = dns.inet.inet_pton(dns.inet.AF_INET6, other.address)
         return cmp(sa, oa)
