@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: resolver.py,v 1.15 2003/07/12 02:29:39 halley Exp $
+# $Id: resolver.py,v 1.20 2003/10/27 00:48:49 halley Exp $
 
 """DNS stub resolver.
 
@@ -40,9 +40,11 @@ class NXDOMAIN(dns.exception.DNSException):
     """The query name does not exist."""
     pass
 
-class Timeout(dns.exception.DNSException):
-    """The query timed out."""
-    pass
+# The definition of the Timeout exception has moved from here to the
+# dns.exception module.  We keep dns.resolver.Timeout defined for
+# backwards compatibility.
+
+Timeout = dns.exception.Timeout
 
 class NoAnswer(dns.exception.DNSException):
     """The response did not contain an answer to the question."""
@@ -310,6 +312,8 @@ class Resolver(object):
         finally:
             if want_close:
                 f.close()
+        if len(self.nameservers) == 0:
+            self.nameservers.append('127.0.0.1')
 
     def _config_win32_nameservers(self, nameservers, split_char=','):
         """Configure a NameServer registry entry."""
@@ -334,22 +338,37 @@ class Resolver(object):
 
     def _config_win32_fromkey(self, key):
         """Extract DNS info from a registry key."""
-        servers, rtype = _winreg.QueryValueEx(key, 'NameServer')
+        try:
+            servers, rtype = _winreg.QueryValueEx(key, 'NameServer')
+        except WindowsError:
+            servers = None
         if servers:
             self._config_win32_nameservers(servers)
-            dom, rtype = _winreg.QueryValueEx(key, 'Domain')
-            if dom:
-                self._config_win32_domain(servers)
+            try:
+                dom, rtype = _winreg.QueryValueEx(key, 'Domain')
+                if dom:
+                    self._config_win32_domain(servers)
+            except WindowsError:
+                pass
         else:
-            servers, rtype = _winreg.QueryValueEx(key, 'DhcpNameServer')
+            try:
+                servers, rtype = _winreg.QueryValueEx(key, 'DhcpNameServer')
+            except WindowsError:
+                servers = None
             if servers:
                 # Annoyingly, the DhcpNameServer list is apparently space
                 # separated instead of comma separated like NameServer.
                 self._config_win32_nameservers(servers, ' ')
-                dom, rtype = _winreg.QueryValueEx(key, 'DhcpDomain')
-                if dom:
-                    self._config_win32_domain(servers)
-        search, rtype = _winreg.QueryValueEx(key, 'SearchList')
+                try:
+                    dom, rtype = _winreg.QueryValueEx(key, 'DhcpDomain')
+                    if dom:
+                        self._config_win32_domain(servers)
+                except WindowsError:
+                    pass
+        try:
+            search, rtype = _winreg.QueryValueEx(key, 'SearchList')
+        except WindowsError:
+            search = None
         if search:
             self._config_win32_search(servers)
 
@@ -388,8 +407,11 @@ class Resolver(object):
                             try:
                                 # enabled interfaces seem to have a non-empty
                                 # NTEContextList
-                                (nte, ttype) = _winreg.QueryValueEx(key,
+                                try:
+                                    (nte, ttype) = _winreg.QueryValueEx(key,
                                                              'NTEContextList')
+                                except WindowsError:
+                                    nte = None
                                 if nte:
                                     self._config_win32_fromkey(key)
                             finally:
