@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: tokenizer.py,v 1.20 2003/07/16 10:42:43 halley Exp $
+# $Id: tokenizer.py,v 1.23 2003/11/01 02:29:41 halley Exp $
 
 """Tokenize DNS master file format"""
 
@@ -54,7 +54,7 @@ class Tokenizer(object):
     I{value} is a string.  The valid types are EOF, EOL, WHITESPACE,
     IDENTIFIER, QUOTED_STRING, COMMENT, and DELIMITER.
     
-    @ivar file: The file tokenize
+    @ivar file: The file to tokenize
     @type file: file
     @ivar ungotten_char: The most recently ungotten character, or None.
     @type ungotten_char: string
@@ -71,19 +71,34 @@ class Tokenizer(object):
     @type eof: bool
     @ivar delimiters: The current delimiter dictionary.
     @type delimiters: dict
+    @ivar line_number: The current line number
+    @type line_number: int
+    @ivar filename: A filename that will be returned by the L{where} method.
+    @type filename: string
     """
     
-    def __init__(self, f=sys.stdin):
+    def __init__(self, f=sys.stdin, filename=None):
         """Initialize a tokenizer instance.
 
         @param f: The file to tokenize.  The default is sys.stdin.
         This parameter may also be a string, in which case the tokenizer
         will take its input from the contents of the string.
         @type f: file or string
+        @param filename: the name of the filename that the L{where} method
+        will return.
+        @type filename: string
         """
         
         if isinstance(f, str):
             f = cStringIO.StringIO(f)
+            if filename is None:
+                filename = '<string>'
+        else:
+            if filename is None:
+                if f is sys.stdin:
+                    filename = '<stdin>'
+                else:
+                    filename = '<file>'
         self.file = f
         self.ungotten_char = None
         self.ungotten_token = None
@@ -91,6 +106,8 @@ class Tokenizer(object):
         self.quoting = False
         self.eof = False
         self.delimiters = _DELIMITERS
+        self.line_number = 1
+        self.filename = filename
 
     def _get_char(self):
         """Read a character from input.
@@ -104,11 +121,22 @@ class Tokenizer(object):
                 c = self.file.read(1)
                 if c == '':
                     self.eof = True
+                elif c == '\n':
+                    self.line_number += 1
         else:
             c = self.ungotten_char
             self.ungotten_char = None
         return c
 
+    def where(self):
+        """Return the current location in the input.
+
+        @rtype: (string, int) tuple.  The first item is the filename of
+        the input, the second is the current line number.
+        """
+        
+        return (self.filename, self.line_number)
+    
     def _unget_char(self, c):
         """Unget a character.
 
@@ -248,8 +276,17 @@ class Tokenizer(object):
                         if not (c2.isdigit() and c3.isdigit()):
                             raise dns.exception.SyntaxError
                         c = chr(int(c) * 100 + int(c2) * 10 + int(c3))
-                elif c== '\n':
+                elif c == '\n':
                     raise dns.exception.SyntaxError, 'newline in quoted string'
+            elif c == '\\':
+                #
+                # Treat \ followed by a delimiter as the 
+                # delimiter, otherwise leave it alone.
+                #
+                c = self._get_char()
+                if c == '' or not c in self.delimiters:
+                    self._unget_char(c)
+                    c = '\\'
             token += c
         if token == '' and ttype != QUOTED_STRING:
             if self.multiline:
